import { Logger } from '@socketsecurity/registry/lib/logger';
import { Remap } from '@socketsecurity/registry/lib/objects';
import { EditablePackageJson } from '@socketsecurity/registry/lib/packages';
import { SemVer } from 'semver';
declare const AGENTS: readonly ["bun", "npm", "pnpm", "yarn/berry", "yarn/classic", "vlt"];
type Agent = (typeof AGENTS)[number];
type StringKeyValueObject = {
    [key: string]: string;
};
type DetectOptions = {
    cwd?: string | undefined;
    onUnknown?: (pkgManager: string | undefined) => void;
};
type EnvBase = {
    agent: Agent;
    agentExecPath: string;
    agentSupported: boolean;
    features: {
        npmBuggyOverrides: boolean;
    };
    nodeSupported: boolean;
    nodeVersion: SemVer;
    npmExecPath: string;
    pkgRequirements: {
        agent: string;
        node: string;
    };
    pkgSupports: {
        agent: boolean;
        node: boolean;
    };
};
type EnvDetails = Readonly<Remap<EnvBase & {
    agentVersion: SemVer;
    lockName: string;
    lockPath: string;
    lockSrc: string;
    pkgJson: EditablePackageJson;
    pkgPath: string;
}>>;
type PartialEnvDetails = Readonly<Remap<EnvBase & {
    agentVersion: SemVer | undefined;
    lockName: string | undefined;
    lockPath: string | undefined;
    lockSrc: string | undefined;
    pkgJson: EditablePackageJson | undefined;
    pkgPath: string | undefined;
}>>;
declare function detectPackageEnvironment({ cwd, onUnknown }?: DetectOptions): Promise<EnvDetails | PartialEnvDetails>;
type DetectAndValidateOptions = {
    cmdName?: string | undefined;
    logger?: Logger | undefined;
    prod?: boolean | undefined;
};
declare function detectAndValidatePackageEnvironment(cwd: string, options?: DetectAndValidateOptions | undefined): Promise<void | EnvDetails>;
export { AGENTS, Agent, StringKeyValueObject, DetectOptions, EnvDetails, PartialEnvDetails, detectPackageEnvironment, DetectAndValidateOptions, detectAndValidatePackageEnvironment };
